@@ -20,7 +20,6 @@
 
 import re
 import logging
-import os
 import sys
 import copy
 import time
@@ -33,7 +32,10 @@ from collections import OrderedDict
 from contextlib import contextmanager
 
 from lava_dispatcher.pipeline.log import YAMLLogger
-from lava_dispatcher.pipeline.utils.constants import OVERRIDE_CLAMP_DURATION, ACTION_TIMEOUT
+from lava_dispatcher.pipeline.utils.constants import (
+    ACTION_TIMEOUT,
+    OVERRIDE_CLAMP_DURATION
+)
 
 if sys.version > '3':
     from functools import reduce  # pylint: disable=redefined-builtin
@@ -121,26 +123,6 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         if not action:
             raise RuntimeError("Unable to add empty action to pipeline")
 
-    def _override_action_timeout(self, action, override):
-        if not isinstance(override, dict):
-            return
-        action.timeout = Timeout(
-            action.name,
-            Timeout.parse(
-                override[action.name]
-            )
-        )
-
-    def _override_connection_timeout(self, action, override):
-        if not isinstance(override, dict):
-            return
-        action.connection_timeout = Timeout(
-            action.name,
-            Timeout.parse(
-                override[action.name]
-            )
-        )
-
     def add_action(self, action, parameters=None):  # pylint: disable=too-many-branches
         self._check_action(action)
         self.actions.append(action)
@@ -154,16 +136,6 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
             action.section = self.parent.section
         else:
             action.level = "%s" % (len(self.actions))
-        # create a log handler just for this action.
-        if self.job and self.job.parameters['output_dir']:
-            log_level_dir = action.level.split('.')[0]
-            yaml_filename = os.path.join(
-                self.job.parameters['output_dir'], log_level_dir,
-                "%s-%s.log" % (action.level, action.name)
-            )
-            if not os.path.exists(os.path.dirname(yaml_filename)):
-                os.makedirs(os.path.dirname(yaml_filename))
-            action.log_filename = yaml_filename
 
         # Use the pipeline parameters if the function was walled without
         # parameters.
@@ -175,17 +147,19 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
             # some action handlers do not need to pass all parameters to their children.
             action.connection_timeout.duration = parameters['default_connection_timeout']
         # Set the timeout
+        # pylint: disable=protected-access
         # FIXME: only the last test is really useful. The first ones are only
         # needed to run the tests that do not use a device and job.
         if self.job is not None and self.job.device is not None:
             # set device level overrides
             overrides = self.job.device.get('timeouts', {})
             if 'actions' in overrides and action.name in overrides['actions']:
-                self._override_action_timeout(action, overrides['actions'])
+                action._override_action_timeout(overrides['actions'])
             elif action.name in overrides:
-                self._override_action_timeout(action, overrides)
+                action._override_action_timeout(overrides)
+                parameters['timeout'] = overrides[action.name]
             if 'connections' in overrides and action.name in overrides['connections']:
-                self._override_connection_timeout(action, overrides['connections'])
+                action._override_connection_timeout(overrides['connections'])
         # Set the parameters after populate so the sub-actions are also
         # getting the parameters.
         # Also set the parameters after the creation of the default timeout
@@ -193,15 +167,16 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         # job overrides device timeouts:
         if self.job and 'timeouts' in self.job.parameters:
             overrides = self.job.parameters['timeouts']
-            if 'actions' in overrides and action.name in overrides:
+            if 'actions' in overrides and action.name in overrides['actions']:
                 # set job level overrides
-                self._override_action_timeout(action, overrides['actions'])
+                action._override_action_timeout(overrides['actions'])
             elif action.name in overrides:
-                self._override_action_timeout(action, overrides)
+                action._override_action_timeout(overrides)
                 parameters['timeout'] = overrides[action.name]
-            if 'connections' in overrides and action.name in overrides:
-                self._override_connection_timeout(action, overrides['connections'])
-
+            elif 'timeout' in parameters:
+                action.timeout.duration = Timeout.parse(parameters['timeout'])
+            if 'connections' in overrides and action.name in overrides['connections']:
+                action._override_connection_timeout(overrides['connections'])
         action.parameters = parameters
 
     def describe(self, verbose=True):
@@ -284,18 +259,6 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         # Diagnosis is not allowed to alter the connection, do not use the return value.
         return None
 
-    def _log_action_results(self, action):
-        if action.results and isinstance(action.logger, YAMLLogger):
-            action.results.update(
-                {
-                    'level': action.level,
-                    'duration': action.elapsed_time,
-                    'timeout': action.timeout.duration,
-                    'connection-timeout': action.connection_timeout.duration
-                }
-            )
-            action.logger.results({action.name: action.results})
-
     def run_actions(self, connection, args=None):  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
 
         def cancelling_handler(*args):  # pylint: disable=unused-argument
@@ -315,8 +278,9 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                 # the action which overran the timeout has been allowed to complete.
                 name = self.job.parameters.get('job_name', '?')
                 msg = "Job '%s' timed out after %s seconds" % (name, int(self.job.timeout.duration))
+                action.logger.error(msg)
                 action.errors = msg
-                final = self.actions[-1]
+                final = self.job.pipeline.actions[-1]
                 if final.name == "finalize":
                     final.run(connection, None)
                 else:
@@ -331,10 +295,9 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
             if isinstance(action.logger, YAMLLogger):
                 action.logger.setMetadata(action.level, action.name)
             # Add action start timestamp to the log message
-            msg = {'msg': 'start: %s %s (max %ds)' % (action.level,
-                                                      action.name,
-                                                      action.timeout.duration),
-                   'ts': datetime.datetime.utcnow().isoformat()}
+            msg = 'start: %s %s (max %ds)' % (action.level,
+                                              action.name,
+                                              action.timeout.duration)
             if self.parent is None:
                 action.logger.info(msg)
             else:
@@ -352,8 +315,8 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                 except (ValueError, KeyError, NameError, SyntaxError, OSError,
                         TypeError, RuntimeError, AttributeError):
                     action.elapsed_time = time.time() - start
-                    msg = re.sub('\s+', ' ', ''.join(traceback.format_exc().split('\n')))
-                    action.logger.exception(msg)
+                    msg = re.sub(r'\s+', ' ', ''.join(traceback.format_exc().split('\n')))
+                    action.logger.exception(traceback.format_exc())
                     action.errors = msg
                     action.cleanup()
                     self.cleanup_actions(connection, None)
@@ -363,14 +326,13 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                     raise KeyboardInterrupt
                 action.elapsed_time = time.time() - start
                 # Add action end timestamp to the log message
-                msg = {'msg': "%s duration: %.02f" % (action.name,
-                                                      action.elapsed_time),
-                       'ts': datetime.datetime.utcnow().isoformat()}
+                msg = "%s duration: %.02f" % (action.name,
+                                              action.elapsed_time)
                 if self.parent is None:
                     action.logger.info(msg)
                 else:
                     action.logger.debug(msg)
-                self._log_action_results(action)
+                action.log_action_results()
                 if new_connection:
                     connection = new_connection
             except KeyboardInterrupt:
@@ -387,7 +349,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                 # set results including retries
                 if "boot-result" not in action.data:
                     action.data['boot-result'] = 'failed'
-                self._log_action_results(action)
+                action.log_action_results()
                 action.logger.exception(str(exc))
                 self._diagnose(connection)
                 action.cleanup()
@@ -395,7 +357,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                 # but the failing action may be inside an internal pipeline of the retry
                 if not self.parent:  # top level pipeline, no retries left
                     self.cleanup_actions(connection, exc_message)
-                raise exc
+                raise
         return connection
 
     def prepare_actions(self):
@@ -407,7 +369,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
             action.post_process()
 
 
-class Action(object):  # pylint: disable=too-many-instance-attributes
+class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-public-methods
 
     def __init__(self):
         """
@@ -426,7 +388,6 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         self.__parameters__ = {}
         self.__errors__ = []
         self.elapsed_time = None
-        self.log_filename = None
         self.job = None
         self.logger = logging.getLogger('dispatcher')
         self.__results__ = OrderedDict()
@@ -558,7 +519,9 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         self.timeout.name = self.name
         # Overide the duration if needed
         if 'timeout' in self.parameters:
-            self.timeout.duration = Timeout.parse(self.parameters['timeout'])
+            # preserve existing overrides
+            if self.timeout.duration == Timeout.default_duration():
+                self.timeout.duration = Timeout.parse(self.parameters['timeout'])
         if 'connection_timeout' in self.parameters:
             self.connection_timeout.duration = Timeout.parse(self.parameters['connection_timeout'])
 
@@ -571,8 +534,8 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
             self.max_retries = self.parameters['repeat']
         if self.job:
             if self.job.device:
-                if 'character-delays' in self.job.device:
-                    self.character_delay = self.job.device['character-delays'].get(self.section, 0)
+                if 'character_delays' in self.job.device:
+                    self.character_delay = self.job.device['character_delays'].get(self.section, 0)
 
     @parameters.setter
     def parameters(self, data):
@@ -605,9 +568,16 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         # Basic checks
         if not self.name:
             self.errors = "%s action has no name set" % self
-        if ' ' in self.name:
+        # have already checked that self.name is not None, but pylint gets confused.
+        if ' ' in self.name:  # pylint: disable=unsupported-membership-test
             self.errors = "Whitespace must not be used in action names, only descriptions or summaries: %s" % self.name
 
+        if not self.summary:
+            self.errors = "action %s (%s) lacks a summary" % (self.name, self)
+
+        if not self.description:
+            self.errors = "action %s (%s) lacks a description" % (self.name, self)
+
         if not self.section:
             self.errors = "%s action has no section set" % self
 
@@ -652,19 +622,22 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         log = None
         # nice is assumed to always exist (coreutils)
         command_list.insert(0, 'nice')
+        self.logger.info("%s", ' '.join(command_list))
         try:
             log = subprocess.check_output(command_list, stderr=subprocess.STDOUT)
-            log = log.decode('utf-8')
+            log = log.decode('utf-8')  # pylint: disable=redefined-variable-type
         except subprocess.CalledProcessError as exc:
             if sys.version > '3':
                 if exc.output:
                     self.errors = exc.output.strip().decode('utf-8')
                 else:
                     self.errors = str(exc)
-                self.logger.exception({
-                    'command': [i.strip() for i in exc.cmd],
-                    'message': str(exc),
-                    'output': str(exc).split('\n')})
+                self.logger.exception(
+                    '[%s] command %s\nmessage %s\noutput %s\n',
+                    self.name,
+                    [i.strip() for i in exc.cmd],
+                    str(exc),
+                    str(exc).split('\n'))
             else:
                 if exc.output:
                     self.errors = exc.output.strip()
@@ -672,18 +645,18 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
                     self.errors = exc.message
                 else:
                     self.errors = str(exc)
-                self.logger.exception({
-                    'command': [i.strip() for i in exc.cmd],
-                    'message': [i.strip() for i in exc.message],
-                    'output': exc.output.split('\n')})
+                self.logger.exception(
+                    "[%s] command %s\nmessage %s\noutput %s\nexit code %s",
+                    self.name,
+                    [i.strip() for i in exc.cmd],
+                    [i.strip() for i in exc.message],
+                    exc.output.split('\n'), exc.returncode)
 
         # allow for commands which return no output
         if not log and allow_silent:
-            self.logger.debug({'command': command_list})
             return self.errors == []
         else:
-            self.logger.debug({'command': command_list,
-                               'output': log})
+            self.logger.debug('command output %s', log)
             return log
 
     def call_protocols(self):
@@ -769,9 +742,12 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         # noinspection PySetFunctionToLiteral
         for attr in attrs - set([
                 'internal_pipeline', 'job', 'logger', 'pipeline',
+                'default_fixupdict', 'pattern',
                 'parameters', 'SignalDirector', 'signal_director']):
             if attr == 'timeout':
                 data['timeout'] = {'duration': self.timeout.duration, 'name': self.timeout.name}
+            elif attr == 'connection_timeout':
+                data['timeout'] = {'duration': self.timeout.duration, 'name': self.timeout.name}
             elif attr == 'url':
                 data['url'] = self.url.geturl()  # pylint: disable=no-member
             elif attr == 'vcs':
@@ -787,6 +763,8 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
                     for protocol_attr in protocol_attrs:
                         if protocol_attr not in ['logger']:
                             data['protocols'][protocol.name][protocol_attr] = getattr(protocol, protocol_attr)
+            elif isinstance(getattr(self, attr), OrderedDict):
+                data[attr] = dict(getattr(self, attr))
             else:
                 data[attr] = getattr(self, attr)
         if 'deployment_data' in self.parameters:
@@ -822,9 +800,57 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         if not connection.connected:
             self.logger.debug("Already disconnected")
             return
-        self.logger.debug("%s: Wait for prompt. %s seconds" % (self.name, int(self.connection_timeout.duration)))
+        self.logger.debug("%s: Wait for prompt %s. %s seconds",
+                          self.name, connection.prompt_str, int(self.connection_timeout.duration))
         return connection.wait()
 
+    def mkdtemp(self):
+        return self.job.mkdtemp(self.name)
+
+    def _override_action_timeout(self, override):
+        """
+        Only to be called by the Pipeline object, add_action().
+        """
+        if not isinstance(override, dict):
+            return
+        self.timeout = Timeout(
+            self.name,
+            Timeout.parse(
+                override[self.name]
+            )
+        )
+
+    def _override_connection_timeout(self, override):
+        """
+        Only to be called by the Pipeline object, add_action().
+        """
+        if not isinstance(override, dict):
+            return
+        self.connection_timeout = Timeout(
+            self.name,
+            Timeout.parse(
+                override[self.name]
+            )
+        )
+
+    def log_action_results(self):
+        if self.results and isinstance(self.logger, YAMLLogger):
+            self.logger.results({
+                "definition": "lava",
+                "case": self.name,
+                "level": self.level,
+                "duration": self.elapsed_time,
+                "result": "fail" if self.errors else "pass",
+                "extra": self.results})
+            self.results.update(
+                {
+                    'level': self.level,
+                    'duration': self.elapsed_time,
+                    'timeout': self.timeout.duration,
+                    'connection-timeout': self.connection_timeout.duration
+                }
+            )
+
 
 class Timeout(object):
     """
