@@ -23,7 +23,8 @@
 
 # pylint: disable=unused-import
 
-from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMU
+from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMU, BootMonitorQemu
+from lava_dispatcher.pipeline.actions.boot.pyocd import BootMonitorPyOCD
 from lava_dispatcher.pipeline.actions.boot.u_boot import UBoot
 from lava_dispatcher.pipeline.actions.boot.kexec import BootKExec
 from lava_dispatcher.pipeline.actions.boot.ssh import SshLogin, Schroot
@@ -33,3 +34,4 @@ from lava_dispatcher.pipeline.actions.boot.lxc import BootLxc
 from lava_dispatcher.pipeline.actions.boot.ipxe import IPXE
 from lava_dispatcher.pipeline.actions.boot.grub import Grub
 from lava_dispatcher.pipeline.actions.boot.iso import BootIsoInstaller
+from lava_dispatcher.pipeline.actions.boot.bootloader_defaults import BootloaderDefaults
