@@ -21,6 +21,7 @@
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
 
+import os.path
 from lava_dispatcher.pipeline.action import (
     Action,
     Pipeline,
@@ -35,6 +36,7 @@ from lava_dispatcher.pipeline.connections.serial import ConnectDevice
 from lava_dispatcher.pipeline.power import ResetDevice
 from lava_dispatcher.pipeline.utils.constants import (
     UBOOT_AUTOBOOT_PROMPT,
+    UBOOT_INTERRUPT_CHARACTER,
     UBOOT_DEFAULT_CMD_TIMEOUT,
     BOOT_MESSAGE,
 )
@@ -96,6 +98,7 @@ class UBootAction(BootAction):
     def populate(self, parameters):
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # customize the device configuration for this job
+        self.internal_pipeline.add_action(UBootPrepareKernelAction())
         self.internal_pipeline.add_action(UBootSecondaryMedia())
         self.internal_pipeline.add_action(UBootCommandOverlay())
         self.internal_pipeline.add_action(ConnectDevice())
@@ -117,7 +120,7 @@ class ExpectBootloaderSession(Action):
         connection = super(ExpectBootloaderSession, self).run(connection, args)
         device_methods = self.job.device['actions']['boot']['methods']
         connection.prompt_str = device_methods['u-boot']['parameters']['bootloader_prompt']
-        self.logger.debug("%s: Waiting for prompt" % self.name)
+        self.logger.debug("%s: Waiting for prompt", self.name)
         self.wait(connection)
         return connection
 
@@ -161,8 +164,12 @@ class UBootRetry(BootAction):
         self.logger.debug(connection.prompt_str)
         connection.timeout = self.connection_timeout
         self.wait(connection)
-        self.logger.error(self.errors)
-        self.data['boot-result'] = 'failed' if self.errors else 'success'
+        # Log an error only when needed
+        if self.errors:
+            self.logger.error(self.errors)
+            self.data['boot-result'] = 'failed'
+        else:
+            self.data['boot-result'] = 'success'
         return connection
 
 
@@ -187,7 +194,7 @@ class UBootInterrupt(Action):
             if self.job.device.connect_command is '':
                 self.errors = "Unable to connect to device %s" % hostname
         else:
-            self.logger.debug("%s may need manual intervention to reboot" % hostname)
+            self.logger.debug("%s may need manual intervention to reboot", hostname)
         device_methods = self.job.device['actions']['boot']['methods']
         if 'bootloader_prompt' not in device_methods['u-boot']['parameters']:
             self.errors = "Missing bootloader prompt for device"
@@ -196,13 +203,21 @@ class UBootInterrupt(Action):
         if not connection:
             raise RuntimeError("%s started without a connection already in use" % self.name)
         connection = super(UBootInterrupt, self).run(connection, args)
-        self.logger.debug("Changing prompt to 'Hit any key to stop autoboot'")
-        # device is to be put into a reset state, either by issuing 'reboot' or power-cycle
-        connection.prompt_str = UBOOT_AUTOBOOT_PROMPT
-        # command = self.job.device['commands'].get('interrupt', '\n')
-        self.wait(connection)
-        connection.sendline(' \n')
         device_methods = self.job.device['actions']['boot']['methods']
+        # device is to be put into a reset state, either by issuing 'reboot' or power-cycle
+        interrupt_prompt = device_methods['u-boot']['parameters'].get('interrupt_prompt', UBOOT_AUTOBOOT_PROMPT)
+        # interrupt_char can actually be a sequence of ASCII characters - sendline does not care.
+        interrupt_char = device_methods['u-boot']['parameters'].get('interrupt_char', UBOOT_INTERRUPT_CHARACTER)
+        # vendor u-boot builds may require one or more control characters
+        interrupt_control_chars = device_methods['u-boot']['parameters'].get('interrupt_ctrl_list', [])
+        self.logger.debug("Changing prompt to '%s'", interrupt_prompt)
+        connection.prompt_str = interrupt_prompt
+        self.wait(connection)
+        if interrupt_control_chars:
+            for char in interrupt_control_chars:
+                connection.sendcontrol(char)
+        else:
+            connection.sendline('%s\n' % interrupt_char)
         connection.prompt_str = device_methods['u-boot']['parameters']['bootloader_prompt']
         self.wait(connection)
         return connection
@@ -235,7 +250,7 @@ class UBootSecondaryMedia(Action):
         if 'boot_part' not in self.parameters:
             self.errors = "Missing boot_part for the partition number of the boot files inside the deployed image"
 
-        self.set_common_data('file', 'kernel', self.parameters['kernel'])
+        self.set_common_data('file', 'kernel', self.parameters.get('kernel', ''))
         self.set_common_data('file', 'ramdisk', self.parameters.get('ramdisk', ''))
         self.set_common_data('file', 'dtb', self.parameters.get('dtb', ''))
         self.set_common_data('uuid', 'root', self.parameters['root_uuid'])
@@ -322,9 +337,15 @@ class UBootCommandOverlay(Action):
         substitutions['{RAMDISK_ADDR}'] = ramdisk_addr
         if not self.get_common_data('tftp', 'ramdisk') and not self.get_common_data('file', 'ramdisk'):
             ramdisk_addr = '-'
-
+        bootcommand = self.parameters['type']
+        if self.parameters['type'] == 'uimage':
+            bootcommand = 'bootm'
+        elif self.parameters['type'] == 'zimage':
+            bootcommand = 'bootz'
+        elif self.parameters['type'] == 'image':
+            bootcommand = 'booti'
         substitutions['{BOOTX}'] = "%s %s %s %s" % (
-            self.parameters['type'], kernel_addr, ramdisk_addr, dtb_addr)
+            bootcommand, kernel_addr, ramdisk_addr, dtb_addr)
 
         substitutions['{RAMDISK}'] = self.get_common_data('file', 'ramdisk')
         substitutions['{KERNEL}'] = self.get_common_data('file', 'kernel')
@@ -342,8 +363,9 @@ class UBootCommandOverlay(Action):
         substitutions['{ROOT}'] = self.get_common_data('uuid', 'root')  # UUID label, not a file
         substitutions['{ROOT_PART}'] = self.get_common_data('uuid', 'boot_part')
 
+        self.data.setdefault('u-boot', {})
         self.data['u-boot']['commands'] = substitute(self.commands, substitutions)
-        self.logger.debug("Parsed boot commands: %s" % '; '.join(self.data['u-boot']['commands']))
+        self.logger.debug("Parsed boot commands: %s", '; '.join(self.data['u-boot']['commands']))
         return connection
 
 
@@ -371,13 +393,89 @@ class UBootCommandsAction(Action):
             self.errors = "%s started without a connection already in use" % self.name
         connection = super(UBootCommandsAction, self).run(connection, args)
         connection.prompt_str = self.params['bootloader_prompt']
-        self.logger.debug("Changing prompt to %s" % connection.prompt_str)
+        self.logger.debug("Changing prompt to %s", connection.prompt_str)
         for line in self.data['u-boot']['commands']:
             self.wait(connection)
             connection.sendline(line, delay=self.character_delay)
         # allow for auto_login
         params = self.job.device['actions']['boot']['methods']['u-boot']['parameters']
         connection.prompt_str = params.get('boot_message', BOOT_MESSAGE)
-        self.logger.debug("Changing prompt to %s" % connection.prompt_str)
+        self.logger.debug("Changing prompt to %s", connection.prompt_str)
         self.wait(connection)
         return connection
+
+
+class UBootPrepareKernelAction(Action):
+    """
+    Convert kernels to uImage or append DTB, if needed
+    """
+    def __init__(self):
+        super(UBootPrepareKernelAction, self).__init__()
+        self.name = "uboot-prepare-kernel"
+        self.description = "convert kernel to uimage or append dtb"
+        self.summary = "prepare/convert kernel"
+        self.type = None
+        self.params = None
+        self.kernel_type = None
+
+    def create_uimage(self, kernel, load_addr, xip, arch, output):  # pylint: disable=too-many-arguments
+        load_addr = int(load_addr, 16)
+        uimage_path = '%s/%s' % (os.path.dirname(kernel), output)
+        if xip:
+            entry_addr = load_addr + 64
+        else:
+            entry_addr = load_addr
+        cmd = "mkimage -A %s -O linux -T kernel" \
+              " -C none -a 0x%x -e 0x%x" \
+              " -d %s %s" % (arch, load_addr,
+                             entry_addr, kernel,
+                             uimage_path)
+        if self.run_command(cmd.split(' ')):
+            return uimage_path
+        else:
+            raise InfrastructureError("uImage creation failed")
+
+    def validate(self):
+        super(UBootPrepareKernelAction, self).validate()
+        self.params = self.job.device['actions']['boot']['methods']['u-boot']['parameters']
+        self.kernel_type = self.get_common_data('type', 'kernel')
+        if 'type' in self.parameters:
+            self.type = str(self.parameters['type']).lower()
+        if self.type:
+            if self.type not in self.job.device['parameters']:
+                self.errors = "Requested kernel boot type '%s' not supported by this device." % self.type
+            if self.type == "bootm" or self.type == "bootz" or self.type == "booti":
+                self.logger.info("booti, bootm and bootz are being deprecated soon, please use 'image','uimage' or 'zimage'")
+        if self.kernel_type:
+            self.kernel_type = str(self.kernel_type).lower()
+            if self.type != self.kernel_type:
+                if 'mkimage_arch' not in self.params:
+                    self.errors = "Missing architecture for uboot mkimage support (mkimage_arch in u-boot parameters)"
+                if self.type == 'zimage' and self.kernel_type == 'uimage':
+                    self.errors = "Can't convert a uimage to zimage"
+                elif self.type == 'zimage' and self.kernel_type == 'image':
+                    self.errors = "Can't convert an image to zimage"
+                elif self.type == 'image' and self.kernel_type == 'zimage':
+                    self.errors = "Can't convert a zimage to image"
+
+    def run(self, connection, args=None):
+        connection = super(UBootPrepareKernelAction, self).run(connection, args)
+        if not self.kernel_type:
+            return connection  # idempotency
+        old_kernel = self.get_common_data('file', 'kernel')
+        filename = self.data['download_action']['kernel']['file']
+        load_addr = self.job.device['parameters'][self.type]['kernel']
+        if 'text_offset' in self.job.device['parameters']:
+            load_addr = self.job.device['parameters']['text_offset']
+        arch = self.params['mkimage_arch']
+        if (self.type == "uimage" or self.type == "bootm") and self.kernel_type == "image":
+            self.logger.debug("Converting image to uimage")
+            self.create_uimage(filename, load_addr, False, arch, 'uImage')
+            new_kernel = os.path.dirname(old_kernel) + '/uImage'
+            self.set_common_data('file', 'kernel', new_kernel)
+        elif (self.type == "uimage" or self.type == "bootm") and self.kernel_type == "zimage":
+            self.logger.debug("Converting zimage to uimage")
+            self.create_uimage(filename, load_addr, False, arch, 'uImage')
+            new_kernel = os.path.dirname(old_kernel) + '/uImage'
+            self.set_common_data('file', 'kernel', new_kernel)
+        return connection
