@@ -30,40 +30,25 @@ from lava_dispatcher.pipeline.action import (
 from lava_dispatcher.pipeline.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.pipeline.utils.constants import (
     LXC_PATH,
-    RAMDISK_FNAME,
-    DISPATCHER_DOWNLOAD_DIR,
+    RAMDISK_FNAME
+)
+from lava_dispatcher.pipeline.utils.installers import (
+    add_late_command,
+    add_to_kickstart
+)
+from lava_dispatcher.pipeline.utils.filesystem import (
+    mkdtemp,
+    prepare_guestfs,
+    copy_in_overlay
 )
-from lava_dispatcher.pipeline.utils.installers import add_late_command
-from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, prepare_guestfs
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.utils.compression import (
     compress_file,
     decompress_file,
     untar_file
 )
-
-
-class ApplyOverlayImage(Action):
-    """
-    Applies the overlay to an image using mntdir
-    * checks that the filesystem we need is actually mounted.
-    """
-    def __init__(self):
-        super(ApplyOverlayImage, self).__init__()
-        self.name = "apply-overlay-image"
-        self.summary = "unpack overlay onto image"
-        self.description = "unpack overlay onto image mountpoint"
-
-    def run(self, connection, args=None):
-        if not self.data['compress-overlay'].get('output'):
-            raise RuntimeError("Unable to find the overlay")
-        if not os.path.ismount(self.data['loop_mount']['mntdir']):
-            raise RuntimeError("Image overlay requested to be applied but %s is not a mountpoint" %
-                               self.data['loop_mount']['mntdir'])
-        connection = super(ApplyOverlayImage, self).run(connection, args)
-        # use tarfile module - no SELinux support here yet
-        untar_file(self.data['compress-overlay'].get('output'), self.data['loop_mount']['mntdir'])
-        return connection
+from lava_dispatcher.pipeline.utils.strings import substitute
+from lava_dispatcher.pipeline.utils.network import dispatcher_ip
 
 
 class ApplyOverlayGuest(Action):
@@ -86,13 +71,39 @@ class ApplyOverlayGuest(Action):
     def run(self, connection, args=None):
         if not self.data['compress-overlay'].get('output'):
             raise RuntimeError("Unable to find the overlay")
-        guest_dir = mkdtemp()
+        guest_dir = self.mkdtemp()
         guest_file = os.path.join(guest_dir, self.guest_filename)
         self.set_common_data('guest', 'filename', guest_file)
         blkid = prepare_guestfs(
             guest_file, self.data['compress-overlay'].get('output'),
             self.job.device['actions']['deploy']['methods']['image']['parameters']['guest']['size'])
         self.results = {'success': blkid}
+        self.set_common_data('guest', 'UUID', blkid)
+        return connection
+
+
+class ApplyOverlayImage(Action):
+
+    def __init__(self):
+        super(ApplyOverlayImage, self).__init__()
+        self.name = "apply-overlay-image"
+        self.summary = "apply overlay to test image"
+        self.description = "apply overlay via guestfs to the test image"
+
+    def validate(self):
+        super(ApplyOverlayImage, self).validate()
+
+    def run(self, connection, args=None):
+        if self.data['compress-overlay'].get('output'):
+            overlay = self.data['compress-overlay'].get('output')
+            self.logger.debug("Overlay: %s", overlay)
+            decompressed_image = self.data['download_action']['image']['file']
+            self.logger.debug("Image: %s", decompressed_image)
+            root_partition = self.parameters['image']['root_partition']
+            self.logger.debug("root_partition: %s", root_partition)
+            copy_in_overlay(decompressed_image, root_partition, overlay)
+        else:
+            self.logger.debug("No overlay to deploy")
         return connection
 
 
@@ -113,8 +124,8 @@ class PrepareOverlayTftp(Action):
         self.internal_pipeline.add_action(ExtractRamdisk())  # idempotent, checks for a ramdisk parameter
         self.internal_pipeline.add_action(ExtractModules())  # idempotent, checks for a modules parameter
         self.internal_pipeline.add_action(ApplyOverlayTftp())
-        self.internal_pipeline.add_action(CompressRamdisk())  # idempotent, checks for a ramdisk parameter
         self.internal_pipeline.add_action(ConfigurePreseedFile())  # idempotent, checks for a preseed parameter
+        self.internal_pipeline.add_action(CompressRamdisk())  # idempotent, checks for a ramdisk parameter
 
     def run(self, connection, args=None):
         connection = super(PrepareOverlayTftp, self).run(connection, args)
@@ -144,20 +155,31 @@ class ApplyOverlayTftp(Action):
         directory = None
         nfs_url = None
         if self.parameters.get('nfsrootfs', None) is not None:
+            if not self.parameters['nfsrootfs'].get('install_overlay', True):
+                self.logger.info("Skipping applying overlay to NFS")
+                return connection
             overlay_file = self.data['compress-overlay'].get('output')
             directory = self.get_common_data('file', 'nfsroot')
             self.logger.info("Applying overlay to NFS")
         elif self.parameters.get('nfs_url', None) is not None:
+            if not self.parameters['nfs_url'].get('install_overlay', True):
+                self.logger.info("Skipping applying overlay to persistent NFS")
+                return connection
             nfs_url = self.parameters.get('nfs_url')
             overlay_file = self.data['compress-overlay'].get('output')
             self.logger.info("Applying overlay to persistent NFS")
             # need to mount the persistent NFS here.
+            # We can't use self.mkdtemp() here because this directory should
+            # not be removed if umount fails.
             directory = mkdtemp(autoremove=False)
             try:
                 subprocess.check_output(['mount', '-t', 'nfs', nfs_url, directory])
             except subprocess.CalledProcessError as exc:
                 raise JobError(exc)
         elif self.parameters.get('ramdisk', None) is not None:
+            if not self.parameters['ramdisk'].get('install_overlay', True):
+                self.logger.info("Skipping applying overlay to ramdisk")
+                return connection
             overlay_file = self.data['compress-overlay'].get('output')
             directory = self.data['extract-overlay-ramdisk']['extracted_ramdisk']
             self.logger.info("Applying overlay to ramdisk")
@@ -167,10 +189,22 @@ class ApplyOverlayTftp(Action):
         else:
             self.logger.debug("No overlay directory")
             self.logger.debug(self.parameters)
-        untar_file(overlay_file, directory)
+        if self.parameters.get('os', None) == "centos_installer":
+            # centos installer ramdisk doesnt like having anything other
+            # than the kickstart config being inserted. Instead, make the
+            # overlay accessible through tftp. Yuck.
+            tftp_dir = os.path.dirname(self.data['download_action']['ramdisk']['file'])
+            shutil.copy(overlay_file, tftp_dir)
+            suffix = self.data['tftp-deploy'].get('suffix', '')
+            self.set_common_data('file', 'overlay', os.path.join(suffix, os.path.basename(overlay_file)))
         if nfs_url:
             subprocess.check_output(['umount', directory])
             os.rmdir(directory)  # fails if the umount fails
+        if overlay_file:
+            untar_file(overlay_file, directory)
+            if nfs_url:
+                subprocess.check_output(['umount', directory])
+                os.rmdir(directory)  # fails if the umount fails
         return connection
 
 
@@ -199,7 +233,7 @@ class ExtractRootfs(Action):  # pylint: disable=too-many-instance-attributes
             return connection
         connection = super(ExtractRootfs, self).run(connection, args)
         root = self.data['download_action'][self.param_key]['file']
-        root_dir = mkdtemp(basedir=DISPATCHER_DOWNLOAD_DIR)
+        root_dir = self.mkdtemp()
         untar_file(root, root_dir)
         self.set_common_data('file', self.file_key, root_dir)
         self.logger.debug("Extracted %s to %s", self.file_key, root_dir)
@@ -228,6 +262,30 @@ class ExtractNfsRootfs(ExtractRootfs):
             self.errors = "no file specified extract as %s" % self.param_key
         if not os.path.exists('/usr/sbin/exportfs'):
             raise InfrastructureError("NFS job requested but nfs-kernel-server not installed.")
+        if 'prefix' in self.parameters[self.param_key]:
+            prefix = self.parameters[self.param_key]['prefix']
+            if prefix.startswith('/'):
+                self.errors = 'prefix must not be an absolute path'
+            if not prefix.endswith('/'):
+                self.errors = 'prefix must be a directory and end with /'
+
+    def run(self, connection, args=None):
+        if not self.parameters.get(self.param_key, None):  # idempotency
+            return connection
+        connection = super(ExtractNfsRootfs, self).run(connection, args)
+
+        if 'prefix' in self.parameters[self.param_key]:
+            prefix = self.parameters[self.param_key]['prefix']
+            self.logger.warning("Adding '%s' prefix, any other content will not be visible.",
+                                prefix)
+
+            # Grab the path already defined in super().run() and add the prefix
+            root_dir = self.get_common_data('file', self.file_key)
+            root_dir = os.path.join(root_dir, prefix)
+            # sets the directory into which the overlay is unpacked and which
+            # is used in the substitutions into the bootloader command string.
+            self.set_common_data('file', self.file_key, root_dir)
+        return connection
 
 
 class ExtractModules(Action):
@@ -258,10 +316,16 @@ class ExtractModules(Action):
         # as the kernel may need some modules to raise the network and
         # will need other modules to support operations within the NFS
         if self.parameters.get('nfsrootfs', None):
+            if not self.parameters['nfsrootfs'].get('install_modules', True):
+                self.logger.info("Skipping applying overlay to NFS")
+                return connection
             root = self.get_common_data('file', 'nfsroot')
             self.logger.info("extracting modules file %s to %s", modules, root)
             untar_file(modules, root)
         if self.parameters.get('ramdisk', None):
+            if not self.parameters['ramdisk'].get('install_modules', True):
+                self.logger.info("Not adding modules to the ramdisk.")
+                return
             root = self.data['extract-overlay-ramdisk']['extracted_ramdisk']
             self.logger.info("extracting modules file %s to %s", modules, root)
             untar_file(modules, root)
@@ -285,20 +349,28 @@ class ExtractRamdisk(Action):
         self.name = "extract-overlay-ramdisk"
         self.summary = "extract the ramdisk"
         self.description = "extract ramdisk to a temporary directory"
+        self.skip = False
 
     def validate(self):
         super(ExtractRamdisk, self).validate()
         if not self.parameters.get('ramdisk', None):  # idempotency
             return
+        if not self.parameters['ramdisk'].get('install_modules', True) and \
+                not self.parameters['ramdisk'].get('install_overlay', True):
+            self.skip = True
+            return
 
     def run(self, connection, args=None):
         if not self.parameters.get('ramdisk', None):  # idempotency
             return connection
+        if self.skip:
+            self.logger.info("Not extracting ramdisk.")
+            return
         connection = super(ExtractRamdisk, self).run(connection, args)
         ramdisk = self.data['download_action']['ramdisk']['file']
-        ramdisk_dir = mkdtemp()
+        ramdisk_dir = self.mkdtemp()
         extracted_ramdisk = os.path.join(ramdisk_dir, 'ramdisk')
-        os.mkdir(extracted_ramdisk)
+        os.mkdir(extracted_ramdisk, 0o755)
         compression = self.parameters['ramdisk'].get('compression', None)
         suffix = ""
         if compression:
@@ -314,13 +386,12 @@ class ExtractRamdisk(Action):
         else:
             # give the file a predictable name
             shutil.move(ramdisk, ramdisk_compressed_data)
-        self.logger.debug(os.system("file %s" % ramdisk_compressed_data))
         ramdisk_data = decompress_file(ramdisk_compressed_data, compression)
         pwd = os.getcwd()
         os.chdir(extracted_ramdisk)
-        cmd = ('cpio -i -F %s' % ramdisk_data).split(' ')
+        cmd = ('cpio -iud -F %s' % ramdisk_data).split(' ')
         if not self.run_command(cmd):
-            raise JobError('Unable to uncompress: %s - missing ramdisk-type?' % ramdisk_data)
+            raise JobError('Unable to extract cpio arcive: %s - missing header definition (i.e. u-boot)?' % ramdisk_data)
         os.chdir(pwd)
         # tell other actions where the unpacked ramdisk can be found
         self.data[self.name]['extracted_ramdisk'] = extracted_ramdisk  # directory
@@ -339,11 +410,16 @@ class CompressRamdisk(Action):
         self.description = "recreate a ramdisk with the overlay applied."
         self.mkimage_arch = None
         self.add_header = None
+        self.skip = False
 
     def validate(self):
         super(CompressRamdisk, self).validate()
         if not self.parameters.get('ramdisk', None):  # idempotency
             return
+        if not self.parameters['ramdisk'].get('install_modules', True) and \
+                not self.parameters['ramdisk'].get('install_overlay', True):
+            self.skip = True
+            return
         if 'parameters' in self.job.device['actions']['deploy']:
             self.add_header = self.job.device['actions']['deploy']['parameters'].get('add_header', None)
             if self.add_header is not None:
@@ -359,6 +435,8 @@ class CompressRamdisk(Action):
     def run(self, connection, args=None):
         if not self.parameters.get('ramdisk', None):  # idempotency
             return connection
+        if self.skip:
+            return connection
         connection = super(CompressRamdisk, self).run(connection, args)
         if 'extracted_ramdisk' not in self.data['extract-overlay-ramdisk']:
             raise RuntimeError("Unable to find unpacked ramdisk")
@@ -366,6 +444,16 @@ class CompressRamdisk(Action):
             raise RuntimeError("Unable to find ramdisk directory")
         ramdisk_dir = self.data['extract-overlay-ramdisk']['extracted_ramdisk']
         ramdisk_data = self.data['extract-overlay-ramdisk']['ramdisk_file']
+        if self.parameters.get('preseed', None):
+            if self.parameters["deployment_data"].get("preseed_to_ramdisk", None):
+                # download action must have completed to get this far
+                # some installers (centos) cannot fetch the preseed file via tftp.
+                # Instead, put the preseed file into the ramdisk using a given name
+                # from deployment_data which we can use in the boot commands.
+                filename = self.parameters["deployment_data"]["preseed_to_ramdisk"]
+                self.logger.info("Copying preseed file into ramdisk: %s", filename)
+                shutil.copy(self.data['download_action']['preseed']['file'], os.path.join(ramdisk_dir, filename))
+                self.set_common_data('file', 'preseed_local', filename)
         pwd = os.getcwd()
         os.chdir(ramdisk_dir)
         self.logger.debug("Building ramdisk %s containing %s",
@@ -373,7 +461,7 @@ class CompressRamdisk(Action):
         cmd = "find . | cpio --create --format='newc' > %s" % ramdisk_data
         try:
             # safe to use shell=True here, no external arguments
-            log = subprocess.check_output(cmd, shell=True)
+            log = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
         except OSError as exc:
             raise RuntimeError('Unable to create cpio filesystem: %s' % exc)
         # lazy-logging would mean that the quoting of cmd causes invalid YAML
@@ -467,3 +555,8 @@ class ConfigurePreseedFile(Action):
         if self.parameters["deployment_data"].get('installer_extra_cmd', None):
             if self.parameters.get('os', None) == "debian_installer":
                 add_late_command(self.data['download_action']['preseed']['file'], self.parameters["deployment_data"]["installer_extra_cmd"])
+            if self.parameters.get('os', None) == "centos_installer":
+                substitutions = {}
+                substitutions['{OVERLAY_URL}'] = 'tftp://' + dispatcher_ip() + '/' + self.get_common_data('file', 'overlay')
+                post_command = substitute([self.parameters["deployment_data"]["installer_extra_cmd"]], substitutions)
+                add_to_kickstart(self.data['download_action']['preseed']['file'], post_command[0])
