@@ -40,6 +40,7 @@ from lava_dispatcher.pipeline.action import (
     Pipeline,
 )
 from lava_dispatcher.pipeline.logical import RetryAction
+from lava_dispatcher.pipeline.utils.compression import untar_file
 from lava_dispatcher.pipeline.utils.constants import (
     FILE_DOWNLOAD_CHUNK_SIZE,
     HTTP_DOWNLOAD_CHUNK_SIZE,
@@ -123,6 +124,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             self.logger.debug("Cleaning up temporary tree.")
             shutil.rmtree(nested_tmp_dir)
         self.data['download_action'][self.key]['file'] = ''
+        super(DownloadHandler, self).cleanup()
 
     def _url_to_fname_suffix(self, path, modify):
         filename = os.path.basename(self.url.path)
@@ -138,13 +140,6 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             filename = os.path.join(path, '.'.join(parts[:-1]))
         return filename, suffix
 
-    def cleanup(self):
-        nested_tmp_dir = os.path.join(self.path, self.key)
-        if os.path.exists(nested_tmp_dir):
-            self.logger.info("%s %s cleanup", self.name, nested_tmp_dir)
-            shutil.rmtree(nested_tmp_dir)
-        super(DownloadHandler, self).cleanup()
-
     @contextlib.contextmanager
     def _decompressor_stream(self):  # pylint: disable=too-many-branches
         dwnld_file = None
@@ -158,14 +153,8 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 compression = self.parameters[self.key].get('compression', False)
 
         fname, _ = self._url_to_fname_suffix(self.path, compression)
-
         if os.path.exists(fname):
-            nested_tmp_dir = os.path.join(self.path, self.key)
-            if os.path.exists(nested_tmp_dir):
-                self.logger.warning("Cleaning up existing directory: %s", nested_tmp_dir)
-                shutil.rmtree(nested_tmp_dir)
-            os.makedirs(nested_tmp_dir)
-            fname = os.path.join(nested_tmp_dir, os.path.basename(fname))
+            os.remove(fname)
 
         decompressor = None
         if compression:
@@ -183,8 +172,21 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             if decompressor:
                 try:
                     buff = decompressor.decompress(buff)
-                except zlib.error as exc:
-                    self.logger.exception(exc)
+                except (EOFError) as exc:
+                    # EOFError can be raised when decompressing a bz2 archive
+                    # generated by pbzip2. If there is something in unsused_data
+                    # try to continue decompression.
+                    if compression == 'bz2':
+                        if decompressor.unused_data:
+                            buff = decompressor.unused_data
+                        else:
+                            self.logger.exception(str(exc))
+                            raise JobError(exc)
+                    else:
+                        self.logger.exception(str(exc))
+                        raise JobError(exc)
+                except (IOError, lzma.error, zlib.error) as exc:
+                    self.logger.exception(str(exc))
                     raise JobError(exc)
             dwnld_file.write(buff)
 
@@ -202,6 +204,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             image = self.parameters['images'][self.key]
             self.url = lavaurl.urlparse(image['url'])
             compression = image.get('compression', None)
+            archive = image.get('archive', None)
             image_name, _ = self._url_to_fname_suffix(self.path, compression)
             image_arg = image.get('image_arg', None)
             overlay = image.get('overlay', False)
@@ -211,7 +214,8 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             self.data['download_action'][self.key]['image_arg'] = image_arg
         else:
             self.url = lavaurl.urlparse(self.parameters[self.key]['url'])
-            compression = self.parameters[self.key].get('compression', False)
+            compression = self.parameters[self.key].get('compression', None)
+            archive = self.parameters[self.key].get('archive', None)
             overlay = self.parameters.get('overlay', False)
             fname, _ = self._url_to_fname_suffix(self.path, compression)
             self.data['download_action'][self.key] = {'file': fname}
@@ -221,6 +225,12 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         if compression:
             if compression not in ['gz', 'bz2', 'xz']:
                 self.errors = "Unknown 'compression' format '%s'" % compression
+        if archive:
+            if archive not in ['tar']:
+                self.errors = "Unknown 'archive' format '%s'" % archive
+        # pass kernel type to boot Action
+        if self.key == 'kernel' and ('kernel' in self.parameters):
+            self.set_common_data('type', self.key, self.parameters[self.key].get('type', None))
 
     def run(self, connection, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
         def progress_unknown_total(downloaded_size, last_value):
@@ -249,7 +259,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             md5sum = remote.get('md5sum', None)
             sha256sum = remote.get('sha256sum', None)
 
-            self.logger.info("downloading %s as %s" % (remote, fname))
+            self.logger.info("downloading %s as %s" % (remote['url'], fname))
 
             downloaded_size = 0
             beginning = time.time()
@@ -285,6 +295,27 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         self.data['download_action'][self.key]['md5'] = md5.hexdigest()
         self.data['download_action'][self.key]['sha256'] = sha256.hexdigest()
 
+        # handle archive files
+        archive = False
+        if 'images' in self.parameters and self.key in self.parameters['images']:
+            archive = self.parameters['images'][self.key].get('archive', False)
+        else:
+            archive = self.parameters[self.key].get('archive', None)
+        if archive:
+            origin = fname
+            target_fname = os.path.basename(origin).rstrip('.' + archive)
+            target_fname_path = os.path.join(os.path.dirname(origin),
+                                             target_fname)
+            if os.path.exists(target_fname_path):
+                os.remove(target_fname_path)
+
+            if archive == 'tar':
+                untar_file(origin, None, member=target_fname,
+                           outfile=target_fname_path)
+                self.data['download_action'][self.key]['file'] = target_fname_path
+                self.set_common_data('file', self.key, target_fname)
+            self.logger.debug("Using %s archive" % archive)
+
         if md5sum is not None:
             if md5sum != self.data['download_action'][self.key]['md5']:
                 self.logger.error("md5sum of downloaded content: %s" % (
@@ -369,9 +400,11 @@ class HttpDownloadAction(DownloadHandler):
     def validate(self):
         super(HttpDownloadAction, self).validate()
         try:
+            self.logger.debug("Validating that %s exists", self.url.geturl())
             res = requests.head(self.url.geturl(), allow_redirects=True, timeout=HTTP_DOWNLOAD_TIMEOUT)
             if res.status_code != requests.codes.OK:  # pylint: disable=no-member
                 # try using (the slower) get for services with broken redirect support
+                self.logger.debug("Using GET because HEAD is not supported properly")
                 res = requests.get(
                     self.url.geturl(), allow_redirects=True, stream=True,
                     timeout=HTTP_DOWNLOAD_TIMEOUT)
