@@ -18,10 +18,9 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-import os
 from lava_dispatcher.pipeline.logical import Deployment
 from lava_dispatcher.pipeline.connections.serial import ConnectDevice
-from lava_dispatcher.pipeline.power import ResetDevice
+from lava_dispatcher.pipeline.power import PowerOn
 from lava_dispatcher.pipeline.action import (
     Pipeline,
     JobError,
@@ -32,10 +31,9 @@ from lava_dispatcher.pipeline.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.pipeline.actions.deploy.download import (
     DownloaderAction,
 )
-from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, copy_to_lxc
+from lava_dispatcher.pipeline.utils.filesystem import copy_to_lxc
 from lava_dispatcher.pipeline.utils.constants import (
-    DISPATCHER_DOWNLOAD_DIR,
-    FASTBOOT_REBOOT_TIMEOUT,
+    FASTBOOT_REBOOT_TIMEOUT
 )
 
 
@@ -94,11 +92,6 @@ class FastbootAction(DeployAction):  # pylint:disable=too-many-instance-attribut
         self.name = "fastboot-deploy"
         self.description = "download files and deploy using fastboot"
         self.summary = "fastboot deployment"
-        self.fastboot_dir = DISPATCHER_DOWNLOAD_DIR
-        try:
-            self.fastboot_dir = mkdtemp(basedir=DISPATCHER_DOWNLOAD_DIR)
-        except OSError:
-            pass
 
     def validate(self):
         super(FastbootAction, self).validate()
@@ -120,40 +113,26 @@ class FastbootAction(DeployAction):  # pylint:disable=too-many-instance-attribut
         if hasattr(self.job.device, 'power_state'):
             if self.job.device.power_state in ['on', 'off']:
                 self.internal_pipeline.add_action(ConnectDevice())
-                self.internal_pipeline.add_action(ResetDevice())
+                self.internal_pipeline.add_action(PowerOn())
         self.internal_pipeline.add_action(EnterFastbootAction())
         self.internal_pipeline.add_action(LxcAddDeviceAction())
+
         image_keys = list(parameters['images'].keys())
-        if 'image' in image_keys:
-            download = DownloaderAction('image', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(FastbootUpdateAction())
-        if 'ptable' in image_keys:
-            download = DownloaderAction('ptable', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(ApplyPtableAction())
-        if 'boot' in image_keys:
-            download = DownloaderAction('boot', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(ApplyBootAction())
-        if 'cache' in image_keys:
-            download = DownloaderAction('cache', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(ApplyCacheAction())
-        if 'userdata' in image_keys:
-            download = DownloaderAction('userdata', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(ApplyUserdataAction())
-        if 'system' in image_keys:
-            download = DownloaderAction('system', self.fastboot_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.internal_pipeline.add_action(ApplySystemAction())
+        fastboot_dir = self.mkdtemp()
+        # Add the required actions
+        checks = [('image', FastbootUpdateAction),
+                  ('ptable', ApplyPtableAction),
+                  ('boot', ApplyBootAction),
+                  ('cache', ApplyCacheAction),
+                  ('userdata', ApplyUserdataAction),
+                  ('system', ApplySystemAction),
+                  ('vendor', ApplyVendorAction)]
+        for (key, cls) in checks:
+            if key in image_keys:
+                download = DownloaderAction(key, fastboot_dir)
+                download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
+                self.internal_pipeline.add_action(download)
+                self.internal_pipeline.add_action(cls())
 
 
 class EnterFastbootAction(DeployAction):
@@ -191,7 +170,7 @@ class EnterFastbootAction(DeployAction):
                    adb_serial_number, 'devices']
         command_output = self.run_command(adb_cmd)
         if command_output and adb_serial_number in command_output:
-            self.logger.debug("Device is in adb: %s" % command_output)
+            self.logger.debug("Device is in adb: %s", command_output)
             adb_cmd = ['lxc-attach', '-n', lxc_name, '--', 'adb',
                        '-s', adb_serial_number, 'reboot-bootloader']
             command_output = self.run_command(adb_cmd)
@@ -205,7 +184,7 @@ class EnterFastbootAction(DeployAction):
                         fastboot_serial_number, 'devices']
         command_output = self.run_command(fastboot_cmd)
         if command_output and fastboot_serial_number in command_output:
-            self.logger.debug("Device is in fastboot: %s" % command_output)
+            self.logger.debug("Device is in fastboot: %s", command_output)
             fastboot_cmd = ['lxc-attach', '-n', lxc_name, '--', 'fastboot',
                             '-s', fastboot_serial_number, 'reboot-bootloader']
             command_output = self.run_command(fastboot_cmd)
@@ -484,3 +463,42 @@ class ApplySystemAction(DeployAction):
             raise JobError("Unable to apply system image using fastboot: %s" %
                            command_output)  # FIXME: JobError needs a unit test
         return connection
+
+
+class ApplyVendorAction(DeployAction):
+    """
+    Fastboot deploy vendor image.
+    """
+
+    def __init__(self):
+        super(ApplyVendorAction, self).__init__()
+        self.name = "fastboot_apply_vendor_action"
+        self.description = "fastboot apply vendor image"
+        self.summary = "fastboot apply vendor"
+        self.retries = 3
+        self.sleep = 10
+
+    def validate(self):
+        super(ApplyVendorAction, self).validate()
+        if 'download_action' not in self.data:
+            raise RuntimeError("download-action missing: %s" % self.name)
+        if 'file' not in self.data['download_action']['vendor']:
+            self.errors = "no file specified for fastboot vendor image"
+        if 'fastboot_serial_number' not in self.job.device:
+            self.errors = "device fastboot serial number missing"
+            if self.job.device['fastboot_serial_number'] == '0000000000':
+                self.errors = "device fastboot serial number unset"
+
+    def run(self, connection, args=None):
+        connection = super(ApplyVendorAction, self).run(connection, args)
+        lxc_name = self.get_common_data('lxc', 'name')
+        src = self.data['download_action']['vendor']['file']
+        dst = copy_to_lxc(lxc_name, src)
+        serial_number = self.job.device['fastboot_serial_number']
+        fastboot_cmd = ['lxc-attach', '-n', lxc_name, '--', 'fastboot',
+                        '-s', serial_number, 'flash', 'vendor', dst]
+        command_output = self.run_command(fastboot_cmd)
+        if command_output and 'error' in command_output:
+            raise JobError("Unable to apply vendor image using fastboot: %s" %
+                           command_output)  # FIXME: JobError needs a unit test
+        return connection
