@@ -152,7 +152,7 @@ class LxcCreateAction(DeployAction):
     def run(self, connection, args=None):
         connection = super(LxcCreateAction, self).run(connection, args)
         if self.lxc_data['lxc_template'] in LXC_TEMPLATE_WITH_MIRROR:
-            lxc_cmd = ['lxc-create', '-t', self.lxc_data['lxc_template'],
+            lxc_cmd = ['lxc-create', '-q', '-t', self.lxc_data['lxc_template'],
                        '-n', self.lxc_data['lxc_name'], '--', '--release',
                        self.lxc_data['lxc_release'], '--arch',
                        self.lxc_data['lxc_arch']]
@@ -166,16 +166,13 @@ class LxcCreateAction(DeployAction):
                             ','.join(self.parameters['packages'])]
             cmd_out_str = 'Generation complete.'
         else:
-            lxc_cmd = ['lxc-create', '-t', self.lxc_data['lxc_template'],
+            lxc_cmd = ['lxc-create', '-q', '-t', self.lxc_data['lxc_template'],
                        '-n', self.lxc_data['lxc_name'], '--', '--dist',
                        self.lxc_data['lxc_distribution'], '--release',
                        self.lxc_data['lxc_release'], '--arch',
                        self.lxc_data['lxc_arch']]
-            cmd_out_str = 'Unpacking the rootfs'
-        command_output = self.run_command(lxc_cmd)
-        if command_output and cmd_out_str not in command_output:
-            raise JobError("Unable to create lxc container: %s" %
-                           command_output)  # FIXME: JobError needs a unit test
+        if not self.run_command(lxc_cmd, allow_silent=True):
+            raise JobError("Unable to create lxc container")
         else:
             self.results = {'status': self.lxc_data['lxc_name']}
         return connection
@@ -199,20 +196,33 @@ class LxcAddDeviceAction(Action):
         connection = super(LxcAddDeviceAction, self).run(connection, args)
         lxc_name = self.get_common_data('lxc', 'name')
         if 'device_path' in list(self.job.device.keys()):
-            # Wait USB_SHOW_UP_TIMEOUT seconds for the usb device to show up
-            self.logger.info("Waiting %d seconds for usb device to show up" %
-                             USB_SHOW_UP_TIMEOUT)
-            sleep(USB_SHOW_UP_TIMEOUT)
+            device_path = self.job.device['device_path']
+            if not isinstance(device_path, list):
+                raise JobError("device_path should be a list")
 
-            device_path = os.path.realpath(self.job.device['device_path'])
-            if os.path.isdir(device_path):
-                devices = os.listdir(device_path)
+            if device_path:
+                # Wait USB_SHOW_UP_TIMEOUT seconds for usb device to show up
+                self.logger.info("[%s] Wait %d seconds for usb device to show up",
+                                 self.name, USB_SHOW_UP_TIMEOUT)
+                sleep(USB_SHOW_UP_TIMEOUT)
+
+                for path in device_path:
+                    path = os.path.realpath(path)
+                    if os.path.isdir(path):
+                        devices = os.listdir(path)
+                    else:
+                        devices = [path]
+
+                    for device in devices:
+                        self.logger.debug('adding %s at %s', device, path)
+                        device = os.path.join(path, device)
+                        lxc_cmd = ['lxc-device', '-n', lxc_name, 'add', device]
+                        log = self.run_command(lxc_cmd)
+                        self.logger.debug(log)
+                        self.logger.debug("%s: devices added from %s", lxc_name,
+                                          path)
             else:
-                devices = [device_path]
-
-            for device in devices:
-                device = os.path.join(device_path, device)
-                lxc_cmd = ['lxc-device', '-n', lxc_name, 'add', device]
-                self.run_command(lxc_cmd)
-            self.logger.debug("%s: devices added from %s", lxc_name,
-                              device_path)
+                self.logger.warning("device_path is None")
+        else:
+            self.logger.error("No device path defined for this device.")
+        return connection
