@@ -29,7 +29,7 @@ from lava_dispatcher.pipeline.actions.deploy.testdef import (
     TestDefinitionAction,
     get_test_action_namespaces,
 )
-from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, check_ssh_identity_file
+from lava_dispatcher.pipeline.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.utils.network import rpcinfo_nfs
 from lava_dispatcher.pipeline.protocols.multinode import MultinodeProtocol
@@ -81,8 +81,13 @@ class OverlayAction(DeployAction):
         self.lava_test_dir = os.path.realpath(
             '%s/../../../lava_test_shell' % os.path.dirname(__file__))
         self.scripts_to_copy = []
+        self.lava_v2_test_dir = os.path.realpath(
+            '%s/../../../pipeline/lava_test_shell' % os.path.dirname(__file__))
+        self.v2_scripts_to_copy = []
         # 755 file permissions
         self.xmod = stat.S_IRWXU | stat.S_IXGRP | stat.S_IRGRP | stat.S_IXOTH | stat.S_IROTH
+        self.target_mac = ''
+        self.target_ip = ''
 
     def validate(self):
         super(OverlayAction, self).validate()
@@ -90,12 +95,31 @@ class OverlayAction(DeployAction):
         # Distro-specific scripts override the generic ones
         distro = self.parameters['deployment_data']['distro']
         distro_support_dir = '%s/distro/%s' % (self.lava_test_dir, distro)
+        lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
+        lava_test_results_dir = lava_test_results_dir % self.job.job_id
+        namespace = self.parameters.get('namespace', None)
+        if namespace:
+            self.action_namespaces.append(namespace)
+            self.set_common_data(namespace, 'lava_test_results_dir',
+                                 lava_test_results_dir)
+            lava_test_sh_cmd = self.parameters['deployment_data']['lava_test_sh_cmd']
+            self.set_common_data(namespace, 'lava_test_sh_cmd',
+                                 lava_test_sh_cmd)
         for script in glob.glob(os.path.join(distro_support_dir, 'lava-*')):
             self.scripts_to_copy.append(script)
+        for script in glob.glob(os.path.join(self.lava_v2_test_dir, 'lava-*')):
+            self.v2_scripts_to_copy.append(script)
         if not self.scripts_to_copy:
             self.errors = "Unable to locate lava_test_shell support scripts."
+        if not self.v2_scripts_to_copy:
+            self.errors = "Unable to update lava_test_shell support scripts."
         if self.job.parameters.get('output_dir', None) is None:
             self.errors = "Unable to use output directory."
+        if 'parameters' in self.job.device:
+            if 'interfaces' in self.job.device['parameters']:
+                if 'target' in self.job.device['parameters']['interfaces']:
+                    self.target_mac = self.job.device['parameters']['interfaces']['target'].get('mac', '')
+                    self.target_ip = self.job.device['parameters']['interfaces']['target'].get('ip', '')
 
     def populate(self, parameters):
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
@@ -114,7 +138,7 @@ class OverlayAction(DeployAction):
         * create test runner directories beneath the temporary location
         * copy runners into test runner directories
         """
-        tmp_dir = mkdtemp()
+        tmp_dir = self.mkdtemp()
         namespace = self.parameters.get('namespace', None)
         if namespace:
             if namespace not in get_test_action_namespaces(self.job.parameters):
@@ -147,6 +171,29 @@ class OverlayAction(DeployAction):
                     fout.write("#!%s\n\n" % shell)
                     fout.write(fin.read())
                     os.fchmod(fout.fileno(), self.xmod)
+        for fname in self.v2_scripts_to_copy:
+            with open(fname, 'r') as fin:
+                foutname = os.path.basename(fname)
+                output_file = '%s/bin/%s' % (lava_path, foutname)
+                self.logger.debug("Updating %s", output_file)
+                with open(output_file, 'w') as fout:
+                    fout.write("#!%s\n\n" % shell)
+                    if foutname == 'lava-target-mac':
+                        fout.write("TARGET_DEVICE_MAC='%s'\n" % self.target_mac)
+                    if foutname == 'lava-target-ip':
+                        fout.write("TARGET_DEVICE_IP='%s'\n" % self.target_ip)
+                    fout.write(fin.read())
+                    os.fchmod(fout.fileno(), self.xmod)
+
+        # Generate the file containing the secrets
+        if 'secrets' in self.job.parameters:
+            self.logger.debug("Creating %s/secrets", lava_path)
+            with open(os.path.join(lava_path, 'secrets'), 'w') as fout:
+                for key, value in self.job.parameters['secrets'].items():
+                    if key == 'yaml_line':
+                        continue
+                    fout.write("%s=%s\n" % (key, value))
+
         connection = super(OverlayAction, self).run(connection, args)
         return connection
 
@@ -163,6 +210,8 @@ class MultinodeOverlayAction(OverlayAction):
         self.lava_multi_node_test_dir = os.path.realpath(
             '%s/../../../lava_test_shell/multi_node' % os.path.dirname(__file__))
         self.lava_multi_node_cache_file = '/tmp/lava_multi_node_cache.txt'
+        self.lava_v2_multi_node_test_dir = os.path.realpath(
+            '%s/../../../pipeline/lava_test_shell/multi_node/' % os.path.dirname(__file__))
         self.role = None
         self.protocol = MultinodeProtocol.name
 
@@ -184,8 +233,11 @@ class MultinodeOverlayAction(OverlayAction):
                 self.errors = "multinode job without a specified role"
             else:
                 self.role = self.job.parameters['protocols'][self.protocol]['role']
+        # FIXME: rationalise all this when the V1 code is removed.
+        for script in glob.glob(os.path.join(self.lava_v2_multi_node_test_dir, 'lava-*')):
+            self.v2_scripts_to_copy.append(script)
 
-    def run(self, connection, args=None):
+    def run(self, connection, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
         if self.role is None:
             self.logger.debug("skipped %s", self.name)
             return connection
@@ -213,7 +265,8 @@ class MultinodeOverlayAction(OverlayAction):
         lava_path = os.path.abspath("%s/%s" % (location, lava_test_results_dir))
         scripts_to_copy = glob.glob(os.path.join(self.lava_multi_node_test_dir, 'lava-*'))
         self.logger.debug(self.lava_multi_node_test_dir)
-        self.logger.debug({"lava_path": lava_path, "scripts": scripts_to_copy})
+        self.logger.debug("lava_path: %s", lava_path)
+        self.logger.debug("scripts to copy %s", scripts_to_copy)
 
         for fname in scripts_to_copy:
             with open(fname, 'r') as fin:
@@ -243,6 +296,19 @@ class MultinodeOverlayAction(OverlayAction):
                         fout.write("LAVA_MULTI_NODE_DEBUG='yes'\n")
                     fout.write(fin.read())
                     os.fchmod(fout.fileno(), self.xmod)
+        for fname in self.v2_scripts_to_copy:
+            with open(fname, 'r') as fin:
+                foutname = os.path.basename(fname)
+                output_file = '%s/bin/%s' % (lava_path, foutname)
+                self.logger.debug("Updating %s", output_file)
+                with open(output_file, 'w') as fout:
+                    fout.write("#!%s\n\n" % shell)
+                    fout.write("LAVA_TEST_BIN='%s/bin'\n" % lava_test_results_dir)
+                    fout.write("LAVA_MULTI_NODE_CACHE='%s'\n" % self.lava_multi_node_cache_file)
+                    # always write out full debug logs
+                    fout.write("LAVA_MULTI_NODE_DEBUG='yes'\n")
+                    fout.write(fin.read())
+                    os.fchmod(fout.fileno(), self.xmod)
         self.call_protocols()
         return connection
 
@@ -264,6 +330,7 @@ class VlandOverlayAction(OverlayAction):
         self.params = {}
         self.sysfs = []
         self.tags = []
+        self.names = []
         self.protocol = VlandProtocol.name
 
     def populate(self, parameters):
@@ -289,20 +356,26 @@ class VlandOverlayAction(OverlayAction):
         # same as the parameters of the protocol itself.
         self.params = self.job.parameters['protocols'][self.protocol]
         device_params = self.job.device['parameters']['interfaces']
+        vprotocol = [vprotocol for vprotocol in self.job.protocols if vprotocol.name == self.protocol][0]
+        # needs to be the configured interface for each vlan.
+        for key, _ in self.params.items():
+            if key == 'yaml_line' or key not in vprotocol.params:
+                continue
+            self.names.append(",".join([key, vprotocol.params[key]['iface']]))
         for interface in device_params:
-            self.sysfs.extend(
+            self.sysfs.append(",".join(
                 [
-                    device_params[interface]['sysfs'],
+                    interface,
                     device_params[interface]['mac'],
-                    interface
-                ]
+                    device_params[interface]['sysfs'],
+                ])
             )
         for interface in device_params:
             if not device_params[interface]['tags']:
                 # skip primary interface
                 continue
             for tag in device_params[interface]['tags']:
-                self.tags.extend([interface, tag])
+                self.tags.append(",".join([interface, tag]))
 
     # pylint: disable=anomalous-backslash-in-string
     def run(self, connection, args=None):
@@ -351,10 +424,17 @@ class VlandOverlayAction(OverlayAction):
                         fout.write(r'LAVA_VLAND_SELF="')
                         for line in self.sysfs:
                             fout.write(r"%s\n" % line)
+                    elif foutname == 'lava-vland-names':
+                        fout.write(r'LAVA_VLAND_NAMES="')
+                        for line in self.names:
+                            fout.write(r"%s\n" % line)
                     elif foutname == 'lava-vland-tags':
                         fout.write(r'LAVA_VLAND_TAGS="')
-                        for line in self.tags:
-                            fout.write(r"%s\n" % line)
+                        if not self.tags:
+                            fout.write(r"\n")
+                        else:
+                            for line in self.tags:
+                                fout.write(r"%s\n" % line)
                     fout.write('"\n\n')
                     fout.write(fin.read())
                     os.fchmod(fout.fileno(), self.xmod)
