@@ -22,15 +22,17 @@
 # imported by the parser to populate the list of subclasses.
 
 import os
+import tempfile
+
 from lava_dispatcher.pipeline.action import Pipeline
 from lava_dispatcher.pipeline.logical import Deployment
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.deploy.download import DownloaderAction
 from lava_dispatcher.pipeline.actions.deploy.apply_overlay import PrepareOverlayTftp
 from lava_dispatcher.pipeline.actions.deploy.environment import DeployDeviceEnvironment
+from lava_dispatcher.pipeline.utils.constants import LINE_SEPARATOR
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
-from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, tftpd_dir
-from lava_dispatcher.pipeline.utils.constants import DISPATCHER_DOWNLOAD_DIR
+from lava_dispatcher.pipeline.utils.filesystem import tftpd_dir
 
 
 def tftp_accept(device, parameters):
@@ -87,18 +89,7 @@ class TftpAction(DeployAction):  # pylint:disable=too-many-instance-attributes
         self.name = "tftp-deploy"
         self.description = "download files and deploy using tftp"
         self.summary = "tftp deployment"
-        self.tftp_dir = tftpd_dir()
-        self.suffix = None
-        try:
-            self.tftp_dir = mkdtemp(basedir=self.tftp_dir)
-        except OSError:
-            # allows for unit tests to operate as normal user.
-            self.suffix = '/'
-        self.download_dir = DISPATCHER_DOWNLOAD_DIR  # used for NFS
-        try:
-            self.download_dir = mkdtemp(basedir=DISPATCHER_DOWNLOAD_DIR)
-        except OSError:
-            pass
+        self.tftp_dir = None
 
     def validate(self):
         super(TftpAction, self).validate()
@@ -110,39 +101,39 @@ class TftpAction(DeployAction):  # pylint:disable=too-many-instance-attributes
             self.errors = "Only one of nfsrootfs or nfs_url can be specified"
         lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
         self.data['lava_test_results_dir'] = lava_test_results_dir % self.job.job_id
-        if self.suffix:
-            self.data[self.name].setdefault('suffix', self.suffix)
-        self.data[self.name].setdefault('suffix', os.path.basename(self.tftp_dir))
+        # Extract the 3 last path elements. See action.mkdtemp()
+        suffix = os.path.join(*self.tftp_dir.split('/')[-2:])
+        self.data[self.name].setdefault('suffix', suffix)
         self.errors = infrastructure_error('in.tftpd')
 
+        # Check that the tmp directory is in the tftpd_dir or in /tmp for the
+        # unit tests
+        tftpd_directory = os.path.realpath(tftpd_dir())
+        tftp_dir = os.path.realpath(self.tftp_dir)
+        tmp_dir = tempfile.gettempdir()
+        if not tftp_dir.startswith(tftpd_directory) and \
+           not tftp_dir.startswith(tmp_dir):
+            self.errors = "tftpd directory is not configured correctly, see /etc/default/tftpd-hpa"
+
+        # allow change of lineseparator on sendline during later stages of boot
+        self.set_common_data(
+            'lineseparator',
+            'os_linesep',
+            self.parameters['deployment_data'].get('line_separator', LINE_SEPARATOR))
+
     def populate(self, parameters):
+        self.tftp_dir = self.mkdtemp()
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.set_common_data('tftp', 'tftp_dir', self.tftp_dir)
-        if 'ramdisk' in parameters:
-            download = DownloaderAction('ramdisk', path=self.tftp_dir)
-            download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-            self.internal_pipeline.add_action(download)
-            self.set_common_data('tftp', 'ramdisk', True)
-        if 'kernel' in parameters:
-            download = DownloaderAction('kernel', path=self.tftp_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
-        if 'dtb' in parameters:
-            download = DownloaderAction('dtb', path=self.tftp_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
-        if 'nfsrootfs' in parameters:
-            download = DownloaderAction('nfsrootfs', path=self.download_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
-        if 'modules' in parameters:
-            download = DownloaderAction('modules', path=self.tftp_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
-        if 'preseed' in parameters:
-            download = DownloaderAction('preseed', path=self.tftp_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
+
+        for key in ['ramdisk', 'kernel', 'dtb', 'nfsrootfs', 'modules', 'preseed']:
+            if key in parameters:
+                download = DownloaderAction(key, path=self.tftp_dir)
+                download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
+                self.internal_pipeline.add_action(download)
+                if key == 'ramdisk':
+                    self.set_common_data('tftp', 'ramdisk', True)
+
         # TftpAction is a deployment, so once the files are in place, just do the overlay
         self.internal_pipeline.add_action(PrepareOverlayTftp())
         self.internal_pipeline.add_action(DeployDeviceEnvironment())
