@@ -45,6 +45,8 @@ class ConnectLxc(Action):
         self.shell_class = ShellCommand
 
     def validate(self):
+        if 'lxc' not in self.job.device['actions']['boot']['methods']:
+            return
         super(ConnectLxc, self).validate()
         self.errors = infrastructure_error('lxc-attach')
         if 'prompts' not in self.parameters:
@@ -52,26 +54,43 @@ class ConnectLxc(Action):
 
     def run(self, connection, args=None):
         lxc_name = self.get_common_data('lxc', 'name')
+        if not lxc_name:
+            self.logger.debug("No LXC device requested")
+            return connection
 
-        # Attach usb device to lxc
         if 'device_path' in list(self.job.device.keys()):
-            # Wait USB_SHOW_UP_TIMEOUT seconds for the usb device to show up
-            self.logger.info("Waiting %d seconds for usb device to show up" %
-                             USB_SHOW_UP_TIMEOUT)
-            sleep(USB_SHOW_UP_TIMEOUT)
+            device_path = self.job.device['device_path']
+            if not isinstance(device_path, list):
+                raise JobError("device_path should be a list")
 
-            device_path = os.path.realpath(self.job.device['device_path'])
-            if os.path.isdir(device_path):
-                devices = os.listdir(device_path)
+            if device_path:
+                # Wait USB_SHOW_UP_TIMEOUT seconds for usb device to show up
+                self.logger.info("[%s] Wait %d seconds for usb device to show up",
+                                 self.name, USB_SHOW_UP_TIMEOUT)
+                sleep(USB_SHOW_UP_TIMEOUT)
+
+                for path in device_path:
+                    path = os.path.realpath(path)
+                    if os.path.isdir(path):
+                        devices = os.listdir(path)
+                    else:
+                        devices = [path]
+
+                    for device in devices:
+                        device = os.path.join(path, device)
+                        if os.path.exists(device):
+                            lxc_cmd = ['lxc-device', '-n', lxc_name, 'add', device]
+                            log = self.run_command(lxc_cmd)
+                            self.logger.debug(log)
+                            self.logger.debug("%s: devices added from %s",
+                                              lxc_name, path)
+                        else:
+                            self.logger.info("%s: skipped adding %s device",
+                                             lxc_name, device)
             else:
-                devices = [device_path]
-
-            for device in devices:
-                device = os.path.join(device_path, device)
-                lxc_cmd = ['lxc-device', '-n', lxc_name, 'add', device]
-                self.run_command(lxc_cmd)
-            self.logger.debug("%s: devices added from %s", lxc_name,
-                              device_path)
+                self.logger.warning("device_path is None")
+        else:
+            self.logger.debug("No device path defined for this device.")
 
         cmd = "lxc-attach -n {0}".format(lxc_name)
         self.logger.info("%s Connecting to device using '%s'", self.name, cmd)
