@@ -60,7 +60,9 @@ class RetryAction(Action):
                     return new_connection
             except (JobError, InfrastructureError, TestError) as exc:
                 self.retries += 1
-                self.errors = "%s failed: %d of %d attempts. '%s'" % (self.name, self.retries, self.max_retries, exc)
+                msg = "%s failed: %d of %d attempts. '%s'" % (self.name, self.retries, self.max_retries, exc)
+                self.logger.error(msg)
+                self.errors = msg
                 time.sleep(self.sleep)
         if not self.valid:
             self.errors = "%s retries failed for %s" % (self.retries, self.name)
@@ -76,7 +78,7 @@ class RetryAction(Action):
         self.run(connection)
 
 
-class DiagnosticAction(Action):  # pylint: disable=abstract-class-not-used
+class DiagnosticAction(Action):
 
     def __init__(self):
         """
@@ -98,11 +100,11 @@ class DiagnosticAction(Action):  # pylint: disable=abstract-class-not-used
         Log the requested diagnostic.
         Raises NotImplementedError if subclass has omitted a trigger classmethod.
         """
-        self.logger.debug("%s diagnostic triggered." % self.trigger())
+        self.logger.debug("%s diagnostic triggered.", self.trigger())
         return connection
 
 
-class AdjuvantAction(Action):  # pylint: disable=abstract-class-not-used
+class AdjuvantAction(Action):
     """
     Adjuvants are associative actions - partners and helpers which can be executed if
     the initial Action determines a particular state.
@@ -135,13 +137,13 @@ class AdjuvantAction(Action):  # pylint: disable=abstract-class-not-used
             return connection
         if self.data[self.key()]:
             self.adjuvant = True
-            self.logger.warning("Adjuvant %s required" % self.name)
+            self.logger.warning("Adjuvant %s required", self.name)
         else:
-            self.logger.debug("Adjuvant %s skipped" % self.name)
+            self.logger.debug("Adjuvant %s skipped", self.name)
         return connection
 
 
-class Deployment(object):  # pylint: disable=abstract-class-not-used
+class Deployment(object):
     """
     Deployment is a strategy class which aggregates Actions
     until the request from the YAML can be validated or rejected.
@@ -205,8 +207,7 @@ class Deployment(object):  # pylint: disable=abstract-class-not-used
                 "No deployment strategy available for the given "
                 "device '%s'. %s" % (device['hostname'], cls))
 
-        willing.sort(key=lambda x: x.priority)
-        willing.reverse()
+        willing.sort(key=lambda x: x.priority, reverse=True)
         return willing[0]
 
 
@@ -247,12 +248,11 @@ class Boot(object):
             )
 
         # higher priority first
-        willing.sort(key=lambda x: x.priority)
-        willing.reverse()
+        willing.sort(key=lambda x: x.priority, reverse=True)
         return willing[0]
 
 
-class LavaTest(object):  # pylint: disable=abstract-class-not-used
+class LavaTest(object):
     """
     Allows selection of the LAVA test method for this job within the parser.
     """
@@ -291,8 +291,7 @@ class LavaTest(object):  # pylint: disable=abstract-class-not-used
             raise NotImplementedError(msg)
 
         # higher priority first
-        willing.sort(key=lambda x: x.priority)
-        willing.reverse()
+        willing.sort(key=lambda x: x.priority, reverse=True)
         return willing[0]
 
 
