@@ -22,7 +22,6 @@
 # imported by the parser to populate the list of subclasses.
 
 
-import yaml
 from lava_dispatcher.pipeline.action import (
     Action,
     Pipeline,
@@ -74,7 +73,7 @@ class RebootDevice(Action):
         if self.job.device.power_state is 'on' and self.job.device.soft_reset_command is not '':
             command = self.job.device['commands']['soft_reset']
             if not self.run_command(command.split(' '), allow_silent=True):
-                raise InfrastructureError("%s command failed" % command)
+                raise InfrastructureError("Command '%s' failed" % command)
             self.results = {"success": self.job.device.power_state}
         else:
             connection = super(RebootDevice, self).run(connection, args)
@@ -150,11 +149,16 @@ class PowerOn(Action):
     def run(self, connection, args=None):
         connection = super(PowerOn, self).run(connection, args)
         if self.job.device.power_state is 'off':
+            if self.job.device.pre_power_command:
+                command = self.job.device.pre_power_command
+                self.logger.info("Running pre power command")
+                if not self.run_command(command.split(' '), allow_silent=True):
+                    raise InfrastructureError("%s failed" % command)
             command = self.job.device.power_command
             if not command:
                 return connection
             if not self.run_command(command.split(' '), allow_silent=True):
-                raise InfrastructureError("%s command failed" % command)
+                raise InfrastructureError("Command '%s' failed" % command)
             self.results = {'success': self.name}
             self.job.device.power_state = 'on'
         return connection
@@ -188,34 +192,6 @@ class LxcStop(Action):
         return connection
 
 
-# FIXME: Unused action, but can give fine grained control.
-class LxcDestroy(Action):
-    """
-    Destroys the lxc container at the end of a job
-    """
-    def __init__(self):
-        super(LxcDestroy, self).__init__()
-        self.name = "lxc_destroy"
-        self.summary = "send destroy command"
-        self.description = "destroy the lxc container"
-
-    def validate(self):
-        super(LxcDestroy, self).validate()
-        self.errors = infrastructure_error('lxc-destroy')
-
-    def run(self, connection, args=None):
-        connection = super(LxcDestroy, self).run(connection, args)
-        lxc_name = self.get_common_data('lxc', 'name')
-        if not lxc_name:
-            return connection
-        lxc_cmd = ['lxc-destroy', '-n', lxc_name]
-        command_output = self.run_command(lxc_cmd)
-        if command_output and command_output is not '':
-            raise JobError("Unable to destroy lxc container: %s" %
-                           command_output)  # FIXME: JobError needs a unit test
-        return connection
-
-
 class PowerOff(Action):
     """
     Turns power off at the end of a job
@@ -233,7 +209,7 @@ class PowerOff(Action):
         if self.job.device.power_state is 'on':  # allow for '' and skip
             command = self.job.device['commands']['power_off']
             if not self.run_command(command.split(' '), allow_silent=True):
-                raise InfrastructureError("%s command failed" % command)
+                raise InfrastructureError("Command '%s' failed" % command)
             self.results = {'status': 'success'}
             self.job.device.power_state = 'off'
         return connection
@@ -270,17 +246,13 @@ class FinalizeAction(Action):
             protocol.finalise_protocol(self.job.device)
         if self.errors:
             self.results = {'status': self.errors}
-            self.logger.debug('status: %s' % self.errors)
+            self.logger.error('status: %s', self.errors)
         elif self.job.pipeline.errors:
             self.results = {'status': "Incomplete"}
             self.errors = "Incomplete"
-            self.logger.error({
-                'Status': 'Incomplete',
-                'Errors': self.job.pipeline.errors})
+            self.logger.error('Status: Incomplete\nErrors %s', self.job.pipeline.errors)
         else:
             self.results = {'success': "Complete"}
-            self.logger.debug("Status: Complete")
-        with open("%s/results.yaml" % self.job.parameters['output_dir'], 'w') as results:
-            results.write(yaml.dump(self.job.pipeline.describe()))
+            self.logger.info("Status: Complete")
         # from meliae import scanner
         # scanner.dump_all_objects('filename.json')
