@@ -89,13 +89,7 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertIn('ramdisk', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('kernel', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('dtb', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
-        # allow root to compare the path (with the mkdtemp added)
-        paths = {action.path for action in tftp.internal_pipeline.actions if hasattr(action, 'path')}
         self.assertNotIn('=', tftpd_dir())
-        self.assertIn(
-            tftpd_dir(),
-            [item for item in paths][0]
-        )
 
     def test_device_d02(self):
         factory = Factory()
@@ -197,7 +191,7 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         for action in job.pipeline.actions:
             action.validate()
             if not action.valid:
-                print(action.errors)
+                raise RuntimeError(action.errors)
             self.assertTrue(action.valid)
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
