@@ -26,7 +26,12 @@ import yaml
 import pexpect
 
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
-from lava_dispatcher.pipeline.action import Pipeline, Action, JobError
+from lava_dispatcher.pipeline.action import (
+    Pipeline,
+    Action,
+    JobError,
+    InfrastructureError,
+)
 from lava_dispatcher.pipeline.test.test_basic import Factory, pipeline_reference
 from lava_dispatcher.pipeline.job import Job
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
@@ -35,6 +40,8 @@ from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.test.test_messages import FakeConnection
 from lava_dispatcher.pipeline.utils.messages import LinuxKernelMessages
+from lava_dispatcher.pipeline.test.test_defs import allow_missing_path, check_missing_path
+from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 
 # pylint: disable=invalid-name
 
@@ -166,7 +173,7 @@ class TestKVMBasicDeploy(unittest.TestCase):  # pylint: disable=too-many-public-
 
     def test_validate(self):
         try:
-            self.job.pipeline.validate_actions()
+            allow_missing_path(self.job.pipeline.validate_actions, self, 'qemu-system-x86_64')
         except JobError as exc:
             self.fail(exc)
         for action in self.job.pipeline.actions:
@@ -231,7 +238,7 @@ class TestKVMQcow2Deploy(unittest.TestCase):  # pylint: disable=too-many-public-
 
     def test_validate(self):
         try:
-            self.job.pipeline.validate_actions()
+            allow_missing_path(self.job.pipeline.validate_actions, self, 'qemu-system-x86_64')
         except JobError as exc:
             self.fail(exc)
         for action in self.job.pipeline.actions:
@@ -283,9 +290,38 @@ class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-pu
             self.job.pipeline.validate_actions()
         except JobError as exc:
             self.fail(exc)
+        except InfrastructureError:
+            pass
         for action in self.job.pipeline.actions:
             self.assertEqual([], action.errors)
 
+    def test_extra_options(self):
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
+        kvm_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/kvm-inline.yaml')
+        with open(kvm_yaml) as sample_job_data:
+            job_data = yaml.load(sample_job_data)
+        device['actions']['boot']['methods']['qemu']['parameters']['extra'] = yaml.load("""
+                  - -smp
+                  - 1
+                  - -global
+                  - virtio-blk-device.scsi=off
+                  - -device virtio-scsi-device,id=scsi
+                  - --append "console=ttyAMA0 root=/dev/vda rw"
+                  """)
+        self.assertIsInstance(device['actions']['boot']['methods']['qemu']['parameters']['extra'][1], int)
+        parser = JobParser()
+        job = parser.parse(yaml.dump(job_data), device, 4212, None, None, None,
+                           output_dir='/tmp/')
+        job.validate()
+        boot_image = [action for action in job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        boot_qemu = [action for action in boot_image.internal_pipeline.actions if action.name == 'boot_qemu_image'][0]
+        qemu = [action for action in boot_qemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
+        self.assertIsInstance(qemu.sub_command, list)
+        [self.assertIsInstance(item, str) for item in qemu.sub_command]
+        self.assertIn('virtio-blk-device.scsi=off', qemu.sub_command)
+        self.assertIn('1', qemu.sub_command)
+        self.assertNotIn(1, qemu.sub_command)
+
     def test_pipeline(self):
         description_ref = pipeline_reference('kvm-inline.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
@@ -310,7 +346,7 @@ class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-pu
         inline_repo.data['test-definition'] = {'overlay_dir': location}
 
         inline_repo.run(None)
-        yaml_file = os.path.join(location, 'tests/0_smoke-tests-inline/inline/smoke-tests-basic.yaml')
+        yaml_file = os.path.join(location, '0/tests/0_smoke-tests-inline/inline/smoke-tests-basic.yaml')
         self.assertTrue(os.path.exists(yaml_file))
         with open(yaml_file, 'r') as f_in:
             testdef = yaml.load(f_in)
@@ -343,6 +379,7 @@ class TestAutoLogin(unittest.TestCase):
 
     def test_autologin_prompt_patterns(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
+        self.job.validate()
 
         bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
@@ -354,6 +391,8 @@ class TestAutoLogin(unittest.TestCase):
         # initialise the first Connection object, a command line shell
         shell_connection = prepare_test_connection()
 
+        self.assertIsNotNone(autologinaction.get_common_data('lineseparator', 'os_linesep'))
+
         # Test the AutoLoginAction directly
         conn = autologinaction.run(shell_connection)
 
@@ -370,7 +409,9 @@ class TestAutoLogin(unittest.TestCase):
                                                           'username': 'root'},
                                            'prompts': ['root@debian:~#']})
 
-        self.assertRaises(JobError, self.job.validate)
+        with self.assertRaises((JobError, InfrastructureError)) as check:
+            self.job.validate()
+            check_missing_path(self, check, 'qemu-system-x86_64')
 
     def test_missing_autologin_void_prompts_list(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -380,7 +421,9 @@ class TestAutoLogin(unittest.TestCase):
 
         autologinaction.parameters.update({'prompts': []})
 
-        self.assertRaises(JobError, self.job.validate)
+        with self.assertRaises((JobError, InfrastructureError)) as check:
+            self.job.validate()
+            check_missing_path(self, check, 'qemu-system-x86_64')
 
     def test_missing_autologin_void_prompts_list_item(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -390,7 +433,9 @@ class TestAutoLogin(unittest.TestCase):
 
         autologinaction.parameters.update({'prompts': ['']})
 
-        self.assertRaises(JobError, self.job.validate)
+        with self.assertRaises((JobError, InfrastructureError)) as check:
+            self.job.validate()
+            check_missing_path(self, check, 'qemu-system-x86_64')
 
     def test_missing_autologin_void_prompts_list_item2(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -400,7 +445,9 @@ class TestAutoLogin(unittest.TestCase):
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#', '']})
 
-        self.assertRaises(JobError, self.job.validate)
+        with self.assertRaises((JobError, InfrastructureError)) as check:
+            self.job.validate()
+            check_missing_path(self, check, 'qemu-system-x86_64')
 
     def test_missing_autologin_prompts_list(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -427,7 +474,9 @@ class TestAutoLogin(unittest.TestCase):
 
         autologinaction.parameters.update({'prompts': ''})
 
-        self.assertRaises(JobError, self.job.validate)
+        with self.assertRaises((JobError, InfrastructureError)) as check:
+            self.job.validate()
+            check_missing_path(self, check, 'qemu-system-x86_64')
 
     def test_missing_autologin_prompts_str(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -435,7 +484,7 @@ class TestAutoLogin(unittest.TestCase):
         bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
-        autologinaction.parameters.update({'prompts': 'root@debian:~#'})
+        autologinaction.parameters.update({'prompts': ['root@debian:~#']})
 
         # initialise the first Connection object, a command line shell
         shell_connection = prepare_test_connection()
@@ -544,6 +593,8 @@ class TestChecksum(unittest.TestCase):
             self.job.pipeline.validate_actions()
         except JobError as exc:
             self.fail(exc)
+        except InfrastructureError as exc:
+            check_missing_path(self, exc, 'qemu-system-x86_64')
         for action in self.job.pipeline.actions:
             self.assertEqual([], action.errors)
 
@@ -582,6 +633,8 @@ class TestKvmUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
         factory = Factory()
         self.job = factory.create_kvm_job('sample_jobs/kvm-uefi.yaml', mkdtemp())
 
+    @unittest.skipIf(infrastructure_error('qemu-system-x86_64'),
+                     'qemu-system-x86_64 not installed')
     def test_uefi_path(self):
         deploy = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
         downloaders = [action for action in deploy.internal_pipeline.actions if action.name == 'download_retry']
@@ -598,3 +651,17 @@ class TestKvmUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.job.validate()
         self.assertIn('-L', execute.sub_command)
         self.assertIn(uefi_dir, execute.sub_command)
+
+
+class TestMonitor(unittest.TestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(TestMonitor, self).setUp()
+        factory = Factory()
+        self.job = factory.create_kvm_job('sample_jobs/qemu-monitor.yaml', mkdtemp())
+
+    def test_qemu_monitor(self):
+        self.assertIsNotNone(self.job)
+        self.assertIsNotNone(self.job.pipeline)
+        self.assertIsNotNone(self.job.pipeline.actions)
+        self.job.validate()
