@@ -92,13 +92,7 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertIn('ramdisk', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('kernel', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('dtb', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
-        # allow root to compare the path (with the mkdtemp added)
-        paths = {action.path for action in tftp.internal_pipeline.actions if hasattr(action, 'path')}
         self.assertNotIn('=', tftpd_dir())
-        self.assertIn(
-            tftpd_dir(),
-            [item for item in paths][0]
-        )
 
     def test_device_bbb(self):
         factory = Factory()
@@ -294,7 +288,7 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
                                output_dir='/tmp/')
         job.validate()
         sample_job_data.close()
-        u_boot_media = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1].internal_pipeline.actions[0]
+        u_boot_media = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1].internal_pipeline.actions[1]
         self.assertIsInstance(u_boot_media, UBootSecondaryMedia)
         self.assertEqual([], u_boot_media.errors)
         self.assertEqual(u_boot_media.parameters['kernel'], '/boot/vmlinuz-3.16.0-4-armmp-lpae')
@@ -356,6 +350,17 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertIn('compression', nfs.parameters['nfsrootfs'])
         self.assertEqual(nfs.parameters['nfsrootfs']['compression'], 'xz')
 
+    def test_prefix(self):
+        factory = Factory()
+        job = factory.create_bbb_job('sample_jobs/bbb-skip-install.yaml')
+        job.validate()
+        tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        nfs = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-nfsrootfs'][0]
+        self.assertIn('prefix', nfs.parameters['nfsrootfs'])
+        self.assertEqual(nfs.parameters['nfsrootfs']['prefix'], 'jessie/')
+        self.assertEqual(nfs.param_key, 'nfsrootfs')
+
 
 class TestOverlayCommands(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
@@ -365,8 +370,33 @@ class TestOverlayCommands(unittest.TestCase):  # pylint: disable=too-many-public
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
         nfs = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-nfsrootfs'][0]
+        ramdisk = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-overlay-ramdisk'][0]
         modules = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-modules'][0]
         overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
+        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'), None)
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'), None)
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('compression'), None)
+        self.assertTrue(ramdisk.parameters['ramdisk'].get('install_modules', True))
+        self.assertTrue(ramdisk.parameters['ramdisk'].get('install_overlay', True))
+        self.assertIsNotNone(modules.parameters.get('ramdisk', None))
+        self.assertIsNotNone(modules.parameters.get('nfsrootfs', None))
+        self.assertIsNotNone(nfs.parameters.get('nfsrootfs', None))
+        self.assertIsNotNone(overlay.parameters.get('nfsrootfs', None))
+        self.assertIsNotNone(overlay.parameters.get('ramdisk', None))
+
+    def test_ramdisk_nfs_nomodules(self):
+        factory = Factory()
+        job = factory.create_bbb_job('sample_jobs/bbb-uinitrd-nfs.yaml')
+        tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        nfs = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-nfsrootfs'][0]
+        ramdisk = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-overlay-ramdisk'][0]
+        modules = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-modules'][0]
+        overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
+        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'), None)
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'), None)
+        self.assertIsNone(ramdisk.parameters['ramdisk'].get('compression'), None)
+        self.assertFalse(ramdisk.parameters['ramdisk'].get('install_overlay', True))
         self.assertIsNotNone(modules.parameters.get('ramdisk', None))
         self.assertIsNotNone(modules.parameters.get('nfsrootfs', None))
         self.assertIsNotNone(nfs.parameters.get('nfsrootfs', None))
