@@ -49,7 +49,7 @@ def compress_file(infile, compression):
         log = subprocess.check_output(cmd, shell=True)
         os.chdir(pwd)
         return "%s.%s" % (infile, compression)
-    except OSError as exc:
+    except (OSError, subprocess.CalledProcessError) as exc:
         raise RuntimeError('unable to compress file %s: %s' % (infile, exc))
 
 
@@ -67,14 +67,22 @@ def decompress_file(infile, compression):
         # safe to use shell=True here, no external arguments
         log = subprocess.check_output(cmd, shell=True)
         return outfile
-    except OSError as exc:
+    except (OSError, subprocess.CalledProcessError) as exc:
         raise RuntimeError('unable to decompress file %s: %s' % (infile, exc))
 
 
-def untar_file(infile, outdir):
+def untar_file(infile, outdir, member=None, outfile=None):
     try:
         tar = tarfile.open(infile)
-        tar.extractall(outdir)
-        tar.close()
+        if member:
+            file_obj = tar.extractfile(member)
+            target = open(outfile, 'wb')
+            target.write(file_obj.read())
+            target.close()
+            file_obj.close()
+            tar.close()
+        else:
+            tar.extractall(outdir)
+            tar.close()
     except tarfile.TarError as exc:
         raise JobError("Unable to unpack %s" % infile)
